from ...case import Case, Category, Warning
from infi.execute import execute_assert_success


class FiberChannelDriverAttributes(Case):
    def run_test(self):
        cmd = "lsdev -c driver -t efscsi -F name"
        output = execute_assert_success(cmd.split()).get_stdout().strip().split("\n")
        for name in output:
            cmd = "lsattr -El {} -a fc_err_recov -F value".format(name)
            output = execute_assert_success(cmd.split()).get_stdout().strip()
            if output != "fast_fail":
                # in order to fix this, we run: chdev -l {name} -a fc_err_recov=fast_fail
                msg = "Error recovery policy of FC driver '{}' is set to '{}' instead of 'fast_fail'"
                raise Warning(msg.format(name, output))

    def get_description(self):
        return "Checking FC HBA driver attributes"

    def get_kb_number(self):
        return "204072102"


def get_category():
    return Category("FC HBA", [FiberChannelDriverAttributes])
