
from infi.vendata.powertools.settings.cases.linux.base_cases import SoftwareDependency, ServiceAutoLoadTask, ServiceStartedTask
from infi.vendata.powertools.settings.cases.linux.base_cases import KernelModuleLoaded
from infi.vendata.powertools.settings.cases.linux.base_cases import MultipathConfigRecommened, MultipathConfigDefaults
from infi.vendata.powertools.settings.cases.linux.base_cases import MultipathConfigLocalExclusions, MultipathConfigInfinidatExclusions
from infi.vendata.powertools.settings.cases.linux.base_cases import FiberChannelDriverProperties, RcModulesScript
from infi.vendata.powertools.settings.cases.linux.base_cases import IO_Scheduler_Rule, IO_Scheduler_AffectedSettings
from infi.vendata.powertools.settings.cases.linux.base_cases import DracutPackage, GenerateInitScript
from infi.vendata.powertools.settings.cases import fc_driver_version, connectivity, devices
from infi.vendata.powertools.settings.case import Category
from infi.pkgmgr import RedHatPackageManager
from infi.sysv_service import RedHatInitService

class SG3Utils(SoftwareDependency):

    required_for_provisioning = True

    def __init__(self, category):
        super(SG3Utils, self).__init__(category, "202404121", "sg3_utils", RedHatPackageManager)

class Parted(SoftwareDependency):

    required_for_provisioning = True

    def __init__(self, category):
        super(Parted, self).__init__(category, "202319272", "parted", RedHatPackageManager)

class MultipathToolsPackage(SoftwareDependency):

    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathToolsPackage, self).__init__(category, "202404021", "device-mapper-multipath", RedHatPackageManager)

class MultipathDaemonAutoStart(ServiceAutoLoadTask):

    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathDaemonAutoStart, self).__init__(category, "202319202", "multipathd",
                                                      "multipathd", RedHatInitService)
class MultipathDaemonStartNow(ServiceStartedTask):

    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathDaemonStartNow, self).__init__(category, "202404141", "multipathd",
                                                      "multipathd", RedHatInitService)

    def run_fix(self):
        from infi.multipathtools.client import DEFAULT_CONF_FILE
        with open(DEFAULT_CONF_FILE, 'a'):
            pass
        super(MultipathDaemonStartNow, self).run_fix()

class MultipathModule(KernelModuleLoaded):
    def __init__(self, category):
        from platform import linux_distribution
        _, version, _ = linux_distribution()
        super(MultipathModule, self).__init__(category, "202319212", "dm_multipath")

def get_categories():
    from platform import linux_distribution
    fc_hba_category = fc_driver_version.get_category()
    fc_hba_category.add_case(FiberChannelDriverProperties)
    fc_hba_category.add_case(RcModulesScript)

    return [Category("SCSI", [SG3Utils, Parted]),
            Category("Multipath", [MultipathToolsPackage, MultipathDaemonAutoStart,
                                   MultipathModule, MultipathDaemonStartNow, MultipathConfigDefaults,
                                   MultipathConfigRecommened, MultipathConfigInfinidatExclusions,
                                   MultipathConfigLocalExclusions]),
            Category("Performance", ([IO_Scheduler_Rule, IO_Scheduler_AffectedSettings])),
            fc_hba_category,
            Category("Boot From SAN", ([] if linux_distribution()[1].startswith("5") else [DracutPackage]) +
                                      [GenerateInitScript]),
            devices.get_category(),
            connectivity.get_category()]
