
from infi.vendata.powertools.settings.cases.linux.base_cases import SoftwareDependency, ServiceAutoLoadTask, ServiceStartedTask
from infi.vendata.powertools.settings.cases.linux.base_cases import KernelModuleLoaded
from infi.vendata.powertools.settings.cases.linux.base_cases import MultipathConfigRecommened, MultipathConfigDefaults
from infi.vendata.powertools.settings.cases.linux.base_cases import MultipathConfigLocalExclusions, MultipathConfigInfinidatExclusions
from infi.vendata.powertools.settings.cases.linux.base_cases import FiberChannelDriverProperties, GenerateInitScript
from infi.vendata.powertools.settings.cases.linux.base_cases import IO_Scheduler_Rule, IO_Scheduler_AffectedSettings
from infi.vendata.powertools.settings.cases import fc_driver_version, connectivity, devices
from infi.vendata.powertools.settings.case import Category
from infi.pkgmgr import SusePackageManager
from infi.sysv_service import SuseInitService

class SG3Utils(SoftwareDependency):

    required_for_provisioning = True

    def __init__(self, category):
        super(SG3Utils, self).__init__(category, "202404121", "sg3_utils", SusePackageManager)

class Parted(SoftwareDependency):

    required_for_provisioning = True

    def __init__(self, category):
        super(Parted, self).__init__(category, "202319272", "parted", SusePackageManager)

class MultipathToolsPackage(SoftwareDependency):

    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathToolsPackage, self).__init__(category, "202404021", "multipath-tools", SusePackageManager)

class MultipathDaemonAutoStart(ServiceAutoLoadTask):

    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathDaemonAutoStart, self).__init__(category, "202319202", "multipathd",
                                                      "multipathd", SuseInitService)

class MultipathBootAutoStart(ServiceAutoLoadTask):

    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathBootAutoStart, self).__init__(category, "202319302", "boot.multipath",
                                                     "boot.multipath", SuseInitService)

class UdevBootAutoStart(ServiceAutoLoadTask):

    required_for_provisioning = True

    def __init__(self, category):
        super(UdevBootAutoStart, self).__init__(category, "202319312", "boot.udev",
                                                "boot.udev", SuseInitService)

class MultipathDaemonStartNow(ServiceStartedTask):

    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathDaemonStartNow, self).__init__(category, "202404141", "multipathd",
                                                      "multipathd", SuseInitService)

class MultipathModule(KernelModuleLoaded):
    def __init__(self, category):
        from platform import linux_distribution
        _, version, _ = linux_distribution()
        super(MultipathModule, self).__init__(category, "202319212", "dm_multipath")

def get_categories():
    fc_hba_category = fc_driver_version.get_category()
    fc_hba_category.add_case(FiberChannelDriverProperties)

    return [Category("SCSI", [SG3Utils, Parted]),
            Category("Multipath", [MultipathToolsPackage, MultipathDaemonAutoStart, MultipathModule,
                                   MultipathDaemonStartNow, MultipathConfigDefaults,
                                   MultipathConfigRecommened, MultipathConfigInfinidatExclusions,
                                   MultipathConfigLocalExclusions]),
            Category("Performance", ([IO_Scheduler_Rule, IO_Scheduler_AffectedSettings])),
            fc_hba_category,
            Category("Boot From SAN", [GenerateInitScript]),
            devices.get_category(),
            connectivity.get_category()]
