
from infi.vendata.powertools.settings.cases.linux.base_cases import SoftwareDependency, ServiceAutoLoadTask, ServiceStartedTask
from infi.vendata.powertools.settings.cases.linux.base_cases import KernelModuleLoaded
from infi.vendata.powertools.settings.cases.linux.base_cases import MultipathConfigRecommened, MultipathConfigDefaults
from infi.vendata.powertools.settings.cases.linux.base_cases import MultipathConfigLocalExclusions, MultipathConfigInfinidatExclusions
from infi.vendata.powertools.settings.cases.linux.base_cases import FiberChannelDriverProperties, GenerateInitScript
from infi.vendata.powertools.settings.cases.linux.base_cases import IO_Scheduler_Rule, IO_Scheduler_AffectedSettings
from infi.vendata.powertools.settings.cases import fc_driver_version, connectivity, devices
from infi.vendata.powertools.settings.case import Category
from infi.pkgmgr import UbuntuPackageManager
from infi.sysv_service import UbuntuInitService

class SG3Utils(SoftwareDependency):

    required_for_provisioning = True

    def __init__(self, category):
        super(SG3Utils, self).__init__(category, "202404121", "sg3-utils", UbuntuPackageManager)

class Parted(SoftwareDependency):

    required_for_provisioning = True

    def __init__(self, category):
        super(Parted, self).__init__(category, "202319272", "parted", UbuntuPackageManager)

class SCSITools(SoftwareDependency):

    required_for_provisioning = True

    def __init__(self, category):
        super(SCSITools, self).__init__(category, "202404131", "scsitools", UbuntuPackageManager)

class MultipathToolsPackage(SoftwareDependency):

    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathToolsPackage, self).__init__(category, "202404021", "multipath-tools", UbuntuPackageManager)

class MultipathToolsBootPackage(SoftwareDependency):

    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathToolsBootPackage, self).__init__(category, "202404031", "multipath-tools-boot", UbuntuPackageManager)

class MultipathDaemonAutoStart(ServiceAutoLoadTask):

    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathDaemonAutoStart, self).__init__(category, "202319202", "multipath-tools",
                                                      "multipathd", UbuntuInitService)
class MultipathDaemonStartNow(ServiceStartedTask):

    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathDaemonStartNow, self).__init__(category, "202404141", "multipath-tools",
                                                      "multipathd", UbuntuInitService)

class MultipathModule(KernelModuleLoaded):
    def __init__(self, category):
        super(MultipathModule, self).__init__(category, "202319212", "multipath")

class MultipathdUdevMonitorPackage(SoftwareDependency):

    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathdUdevMonitorPackage, self).__init__(category, "202404181",
                                                           "multipathd-udev-monitor", UbuntuPackageManager)

class MultipathdUdevMonitorAutoStart(ServiceAutoLoadTask):

    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathdUdevMonitorAutoStart, self).__init__(category, "202404191", "multipathd-udev-monitor",
                                                      "multipathd_udev_monitor", UbuntuInitService)

class MultipathdUdevMonitorStartNow(ServiceStartedTask):

    required_for_provisioning = True

    def __init__(self, category):
        super(MultipathdUdevMonitorStartNow, self).__init__(category, "202319292", "multipathd-udev-monitor",
                                                      "multipathd_udev_monitor", UbuntuInitService)

def get_categories():
    from infi.vendata.powertools.utils import get_distribution

    fc_hba_category = fc_driver_version.get_category()
    fc_hba_category.add_case(FiberChannelDriverProperties)

    categories = [Category("SCSI", [SG3Utils, SCSITools, Parted]),
            Category("Multipath", [MultipathToolsPackage, MultipathToolsBootPackage, MultipathDaemonAutoStart,
                                   MultipathModule, MultipathDaemonStartNow, MultipathConfigDefaults,
                                   MultipathConfigRecommened, MultipathConfigInfinidatExclusions,
                                   MultipathConfigLocalExclusions]),
            Category("Performance", ([IO_Scheduler_Rule, IO_Scheduler_AffectedSettings])),
            fc_hba_category,
            Category("Boot From SAN", [GenerateInitScript]),
            devices.get_category(),
            connectivity.get_category()]

    version_specific_categories = {
        "14.04": [Category("Multipathd Udev Monitor", [MultipathdUdevMonitorPackage,
                                                        MultipathdUdevMonitorAutoStart,
                                                        MultipathdUdevMonitorStartNow])]
    }

    categories += version_specific_categories.get(get_distribution().version, [])

    return categories
